@@ -9,8 +9,10 @@
  */
 
 /*
- * "Didst thou think that the eyes of the White Tower were blind?  Nay, I
- * have seen more than thou knowest, Gray Fool."  --Denethor
+ * 'Didst thou think that the eyes of the White Tower were blind?  Nay,
+ *  I have seen more than thou knowest, Grey Fool.'        --Denethor
+ *
+ *     [p.853 of _The Lord of the Rings_, V/vii: "The Pyre of Denethor"]
  */
 
 /*
@@ -29,6 +31,7 @@ Perl_deb_nocontext(const char *pat, ...)
 #ifdef DEBUGGING
     dTHX;
     va_list args;
+    PERL_ARGS_ASSERT_DEB_NOCONTEXT;
     va_start(args, pat);
     vdeb(pat, &args);
     va_end(args);
@@ -42,6 +45,7 @@ void
 Perl_deb(pTHX_ const char *pat, ...)
 {
     va_list args;
+    PERL_ARGS_ASSERT_DEB;
     va_start(args, pat);
 #ifdef DEBUGGING
     vdeb(pat, &args);
@@ -55,22 +59,18 @@ void
 Perl_vdeb(pTHX_ const char *pat, va_list *args)
 {
 #ifdef DEBUGGING
+    dVAR;
     const char* const file = PL_curcop ? OutCopFILE(PL_curcop) : "<null>";
     const char* const display_file = file ? file : "<free>";
     const long line = PL_curcop ? (long)CopLINE(PL_curcop) : 0;
 
-#ifdef USE_5005THREADS
-    PerlIO_printf(Perl_debug_log, "0x%"UVxf" (%s:%ld)\t",
-		  PTR2UV(thr),
-		  (file ? file : "<free>"),
-		  (long)CopLINE(PL_curcop));
-#else
+    PERL_ARGS_ASSERT_VDEB;
+
     if (DEBUG_v_TEST)
 	PerlIO_printf(Perl_debug_log, "(%ld:%s:%ld)\t",
 		      (long)PerlProc_getpid(), display_file, line);
     else
 	PerlIO_printf(Perl_debug_log, "(%s:%ld)\t", display_file, line);
-#endif /* USE_5005THREADS */
     (void) PerlIO_vprintf(Perl_debug_log, pat, *args);
 #else
     PERL_UNUSED_CONTEXT;
@@ -83,6 +83,7 @@ I32
 Perl_debstackptrs(pTHX)
 {
 #ifdef DEBUGGING
+    dVAR;
     PerlIO_printf(Perl_debug_log,
 		  "%8"UVxf" %8"UVxf" %8"IVdf" %8"IVdf" %8"IVdf"\n",
 		  PTR2UV(PL_curstack), PTR2UV(PL_stack_base),
@@ -112,8 +113,12 @@ S_deb_stack_n(pTHX_ SV** stack_base, I32 stack_min, I32 stack_max,
 	I32 mark_min, I32 mark_max)
 {
 #ifdef DEBUGGING
+    dVAR;
     register I32 i = stack_max - 30;
     const I32 *markscan = PL_markstack + mark_min;
+
+    PERL_ARGS_ASSERT_DEB_STACK_N;
+
     if (i < stack_min)
 	i = stack_min;
     
@@ -159,6 +164,7 @@ I32
 Perl_debstack(pTHX)
 {
 #ifndef SKIP_DEBUGGING
+    dVAR;
     if (CopSTASH_eq(PL_curcop, PL_debstash) && !DEBUG_J_TEST_)
 	return 0;
 
@@ -198,6 +204,7 @@ void
 Perl_deb_stack_all(pTHX)
 {
 #ifdef DEBUGGING
+    dVAR;
     I32 si_ix;
     const PERL_SI *si;
 
@@ -234,9 +241,8 @@ Perl_deb_stack_all(pTHX)
 		 */
 
 		I32 i, stack_min, stack_max, mark_min, mark_max;
-		I32 ret_min, ret_max;
 		const PERL_CONTEXT *cx_n = NULL;
-		const PERL_SI      *si_n;
+		const PERL_SI *si_n;
 
 		/* there's a separate stack per SI, so only search
 		 * this one */
@@ -283,27 +289,25 @@ Perl_deb_stack_all(pTHX)
 		}
 
 		mark_min  = cx->blk_oldmarksp;
-		ret_min   = cx->blk_oldretsp;
 		if (cx_n) {
 		    mark_max  = cx_n->blk_oldmarksp;
-		    ret_max   = cx_n->blk_oldretsp;
 		}
 		else {
 		    mark_max = PL_markstack_ptr - PL_markstack;
-		    ret_max  = PL_retstack_ix;
 		}
 
 		deb_stack_n(AvARRAY(si->si_stack),
 			stack_min, stack_max, mark_min, mark_max);
 
-		if (ret_max > ret_min) {
+		if (CxTYPE(cx) == CXt_EVAL || CxTYPE(cx) == CXt_SUB
+			|| CxTYPE(cx) == CXt_FORMAT)
+		{
+		    const OP * const retop = cx->blk_sub.retop;
+
 		    PerlIO_printf(Perl_debug_log, "  retop=%s\n",
-			    PL_retstack[ret_min]
-				? OP_NAME(PL_retstack[ret_min])
-				: "(null)"
+			    retop ? OP_NAME(retop) : "(null)"
 		    );
 		}
-
 	    }
 	} /* next context */
 
@@ -326,8 +330,8 @@ Perl_deb_stack_all(pTHX)
  * Local variables:
  * c-indentation-style: bsd
  * c-basic-offset: 4
- * indent-tabs-mode: t
+ * indent-tabs-mode: nil
  * End:
  *
- * ex: set ts=8 sts=4 sw=4 noet:
+ * ex: set ts=8 sts=4 sw=4 et:
  */
