@@ -296,8 +296,6 @@ PERLVAR(I, sawampersand, bool)		/* must save all match strings */
 PERLVAR(I, unsafe,	bool)
 PERLVAR(I, exit_flags,	U8)		/* was exit() unexpected, etc. */
 
-PERLVARI(I, reginterp_cnt, I32,	 0)	/* Whether "Regexp" was interpolated. */
-
 PERLVAR(I, inplace,	char *)
 PERLVAR(I, e_script,	SV *)
 
@@ -690,6 +688,9 @@ PERLVAR(I, regex_padav,   AV *)		/* All regex objects, indexed via the
 					   Entry 0 is an SV whose PV is a
 					   "packed" list of IVs listing
 					   the now-free slots in the array */
+PERLVAR(I, stashpad,    HV **)		/* for CopSTASH */
+PERLVARI(I, stashpadmax, PADOFFSET, 64)
+PERLVARI(I, stashpadix, PADOFFSET, 0)
 #endif
 
 #ifdef USE_REENTRANT_API
