@@ -247,7 +247,6 @@ unless ($define{'DEBUGGING'}) {
 		    Perl_pad_sv
 		    Perl_pad_setsv
 		    Perl_hv_assert
-		    PL_block_type
 		    PL_watchaddr
 		    PL_watchok
 		    PL_watch_pvx
@@ -347,6 +346,7 @@ unless ($define{'USE_ITHREADS'}) {
 
 unless ($define{'USE_ITHREADS'}) {
     ++$skip{$_} foreach qw(
+		    PL_check_mutex
 		    PL_op_mutex
 		    PL_regex_pad
 		    PL_regex_padav
@@ -354,6 +354,10 @@ unless ($define{'USE_ITHREADS'}) {
 		    PL_hints_mutex
 		    PL_my_ctx_mutex
 		    PL_perlio_mutex
+		    PL_stashpad
+		    PL_stashpadix
+		    PL_stashpadmax
+		    Perl_alloccopstash
 		    Perl_clone_params_del
 		    Perl_clone_params_new
 		    Perl_parser_dup
@@ -421,10 +425,6 @@ unless ($define{'PERL_DEBUG_READONLY_OPS'}) {
 			 );
 }
 
-unless ($define{'THREADS_HAVE_PIDS'}) {
-    ++$skip{PL_ppid};
-}
-
 unless ($define{'PERL_NEED_APPCTX'}) {
     ++$skip{PL_appctx};
 }
