@@ -3,8 +3,11 @@
 # Regenerate (overwriting only if changed):
 #
 #    mg_vtable.h
+#    pod/perlguts.pod
 #
-# from information stored in this file.
+# from information stored in this file.  pod/perlguts.pod is not completely
+# regenerated.  Only the magic table is replaced; the other parts remain
+# untouched.
 #
 # Accepts the standard regen_lib -q and -v args.
 #
@@ -18,16 +21,13 @@ BEGIN {
     require 'regen/regen_lib.pl';
 }
 
-# This generates the relevant section to paste into perlguts.pod to STDOUT
-my $output_guts = grep { $_ eq '-g' } @ARGV;
-
 my %mg =
     (
      sv => { char => '\0', vtable => 'sv', readonly_acceptable => 1,
 	     desc => 'Special scalar variable' },
-     overload => { char => 'A', vtable => 'amagic', desc => '%OVERLOAD hash' },
-     overload_elem => { char => 'a', vtable => 'amagicelem',
-			desc => '%OVERLOAD hash element' },
+     # overload, or type "A" magic, used to be here.  Hence overloaded is
+     # often called AMAGIC internally, even though it does not use "A"
+     # magic any more.
      overload_table => { char => 'c', vtable => 'ovrld',
 			 desc => 'Holds overload table (AMT) on stash' },
      bm => { char => 'B', vtable => 'regexp', value_magic => 1,
@@ -42,8 +42,6 @@ my %mg =
 		  desc => '%ENV hash element' },
      fm => { char => 'f', vtable => 'regdata', value_magic => 1,
 	     readonly_acceptable => 1, desc => "Formline ('compiled' format)" },
-     study => { char => 'G', vtable => 'regexp', value_magic => 1,
-		readonly_acceptable => 1, desc => 'study()ed string' },
      regex_global => { char => 'g', vtable => 'mglob', value_magic => 1,
 		       readonly_acceptable => 1, desc => 'm//g target' },
      hints => { char => 'H', vtable => 'hints', desc => '%^H hash' },
@@ -84,7 +82,7 @@ my %mg =
 		    unknown_to_sv_magic => 1 },
      vec => { char => 'v', vtable => 'vec', value_magic => 1,
 	      desc => 'vec() lvalue' },
-     vstring => { char => 'V', value_magic => 1,
+     vstring => { char => 'V', value_magic => 1, vtable => 'vstring',
 		  desc => 'SV was vstring literal' },
      utf8 => { char => 'w', vtable => 'utf8', value_magic => 1,
 	       desc => 'Cached UTF-8 information' },
@@ -105,7 +103,7 @@ my %mg =
      arylen_p => { char => '@', value_magic => 1,
 		   desc => 'to move arylen out of XPVAV' },
      ext => { char => '~', desc => 'Available for use by extensions' },
-     checkcall => { char => ']', value_magic => 1,
+     checkcall => { char => ']', value_magic => 1, vtable => 'checkcall',
 		    desc => 'inlining/mutation of call to this CV'},
 );
 
@@ -135,8 +133,6 @@ my %sig =
      'regexp' => {set => 'setregexp', alias => [qw(bm fm)]},
      'regdata' => {len => 'regdata_cnt'},
      'regdatum' => {get => 'regdatum_get', set => 'regdatum_set'},
-     'amagic' => {set => 'setamagic', free => 'setamagic'},
-     'amagicelem' => {set => 'setamagic', free => 'setamagic'},
      'backref' => {free => 'killbackrefs'},
      'ovrld' => {free => 'freeovrld'},
      'utf8' => {set => 'setutf8'},
@@ -144,12 +140,15 @@ my %sig =
 		    cond => '#ifdef USE_LOCALE_COLLATE'},
      'hintselem' => {set => 'sethint', clear => 'clearhint'},
      'hints' => {clear => 'clearhints'},
+     'vstring' => {set => 'setvstring'},
+     'checkcall' => {copy => 'copycallchecker'},
 );
 
 my ($vt, $raw, $names) = map {
     open_new($_, '>',
 	     { by => 'regen/mg_vtable.pl', file => $_, style => '*' });
 } 'mg_vtable.h', 'mg_raw.h', 'mg_names.c';
+my $guts = open_new("pod/perlguts.pod", ">");
 
 print $vt <<'EOH';
 /* These constants should be used in preference to raw characters
@@ -215,7 +214,7 @@ EOH
 		     $data->{vtable} ? "vtbl_$data->{vtable}" : '(none)',
 		     $data->{desc}];
     }
-    if ($output_guts) {
+    select +(select($guts), do {
 	my @header = ('(old-style char and macro)', 'MGVTBL', 'Type of magic');
 	my @widths = (0, 0);
 	foreach my $row (@rows) {
@@ -224,12 +223,20 @@ EOH
 		    if length $row->[$_] > $widths[$_];
 	    }
 	}
-	my $indent = '    ';
+	my $indent = ' ';
 	my $format
 	    = sprintf "$indent%%-%ds%%-%ds%%s\n", $widths[0] + 1, $widths[1] + 1;
-	my $desc_wrap = 80 - (length $indent) - $widths[0] - $widths[1] - 2;
+	my $desc_wrap =
+	    79 - 7 - (length $indent) - $widths[0] - $widths[1] - 2;
 
-	print $indent . "mg_type\n";
+	open my $oldguts, "<", "pod/perlguts.pod"
+	   or die "$0 cannot open pod/perlguts.pod for reading: $!";
+	while (<$oldguts>) {
+	    print;
+	    last if /^=for mg_vtable.pl begin/
+	}
+
+	print "\n", $indent . "mg_type\n";
 	printf $format, @header;
 	printf $format, map {'-' x length $_} @header;
 	foreach (@rows) {
@@ -248,7 +255,13 @@ EOH
 	    printf $format, $type, $vtbl, $desc;
 	    printf $format, '', '', $_ foreach @cont;
 	}
-    }
+	print "\n";
+
+	while (<$oldguts>) {
+	    last if /^=for mg_vtable.pl end/;
+	}
+	do { print } while <$oldguts>;
+    })[0];
 }
 
 my @names = sort keys %sig;
@@ -341,3 +354,4 @@ print $vt "#define PL_vtbl_$_ PL_magic_vtables[want_vtbl_$_]\n"
 die "Too many vtable names" if @vtable_names > 63;
 
 read_only_bottom_close_and_rename($_) foreach $vt, $raw, $names;
+		 close_and_rename($guts);
